!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Given vector position returns force.

!latex \item The force vector, ${\bf F}(\boldxi)$, is a combination of the pressure-imbalance Fourier harmonics, $([[p+B^2/2]]\sqrt g^i)_{m,n}$, 
!latex where $i\equiv$\verb+Iusesg+.
!latex and the spectral condensation constraints arising from minimization of $M=\sum_j(m_j^p+n_j^q)(R_j^2+Z_j^2)/2$ with respect to tangential variations,
!latex $\delta R\equiv R_\theta \delta u$ and $\delta Z \equiv Z_\theta \delta u$.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine fc00aa( lNgdof , xx , df , lderiv )

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, half, one
  use numerical, only : machprec
  use fileunits, only : ounit
  use inputlist, only : Wfc00aa, ext, Nvol, Ntor, Lsetmupf, pres, pscale, mu, beta, forceerr, Nquad
  use cputiming
  use allglobal, only : numprocs, myid, cpus, ll, Ngdof, mn, im, in, Rbc, Zbs, Bdmn, Idmn, Bomn, Iomn, mBomn, mIomn, volume, FF, aijkl, lextrap

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS

  INTEGER, intent(in)    :: lNgdof       ! dimension;
  REAL,    intent(in)    :: xx(1:lNgdof) ! vector, representing degrees-of-freedom = internal geometry;
  REAL,    intent(out)   :: df(1:lNgdof) ! force;
  LOGICAL, intent(in)    :: lderiv   ! not presently used; 

  INTEGER                :: liota
  INTEGER                :: llmodnp, ierror, imn, jmn, kmn, lmn, idof
  REAL                   :: lbeta(0:3)

  BEGIN(fc00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  FATALMESS(fc00aa,lNgdof.ne.Ngdof,array dimensions inconsistent)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item The vector, $\boldxi$, represents the geometrical degrees of freedom of the internal interfaces.
!latex This vector is `unpacked' and the \verb+Rbc+, \verb+Zbs+ arrays are assigned.

  do ll = 1,Nvol-1 ; Rbc(ll,1:mn) = xx((ll-1)*(mn+mn-1)+1   :(ll-1)*(mn+mn-1)+mn     ) ! unpack coordinate degrees-of-freedom vector;
                     Zbs(ll,2:mn) = xx((ll-1)*(mn+mn-1)+mn+1:(ll-1)*(mn+mn-1)+mn+mn-1)
  enddo

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  CALL(ex00aa,(lextrap)) !latex \item The routine \verb+ex00aa+ is called to extrapolate innermost surface / magnetic axis.

  CALL(ih00aa,(Nvol)) !latex \item The routine \verb+ih00aa+ is called to interpolate the coordinate harmonics and construct the global coordinates.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item The following routines are called in parallel: \begin{enumerate}

  do ll = 1 , Nvol ! construct Beltrami fields, etc. in parallel;
   
   if( myid.ne.modulo(ll-1,numprocs) ) cycle


   if( Wfc00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc00aa : "f10.2" : myid="i3" ; ll="i3" ; calling ma00ab ; ")')cput-cpus,myid,ll
   endif
   
!latex \item \verb+ma00ab+ : allocates sub-grid geometric arrays, \verb+igss+, etc. in each annulus;
   CALL(ma00ab)
   
   if( Wfc00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc00aa : "f10.2" : myid="i3" ; ll="i3" ; called  ma00ab ;")')cput-cpus,myid,ll
   endif


   if( Wfc00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc00aa : "f10.2" : myid="i3" ; ll="i3" ; calling ma00aa ;")')cput-cpus,myid,ll
   endif

!latex \item \verb+ma00aa+ : calls \verb+me00ab+ on sub-sub-grid to compute matrix elements on sub-grid; calculates \verb+igss+ etc.;
   CALL(ma00aa,(Nquad))

   if( Wfc00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc00aa : "f10.2" : myid="i3" ; ll="i3" ; called  ma00aa ;")')cput-cpus,myid,ll
   endif


   if( Wfc00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc00aa : "f10.2" : myid="i3" ; ll="i3" ; calling ma02aa ;")')cput-cpus,myid,ll
   endif

!latex \item \verb+ma02aa+ : solve Beltrami fields in each annulus (via call to \verb+mp00aa+); adjusts $\mu$, $\psi_p$ to match transform constraints;
   liota = Lsetmupf ; CALL(ma02aa,(liota,ierror))

   if( Wfc00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc00aa : "f10.2" : myid="i3" ; ll="i3" ; called  ma02aa ;")')cput-cpus,myid,ll
   endif


   Bdmn(0,1:mn) = zero ; Bdmn(1,1:mn) = one ; Idmn(2:mn) = one ! dummy `default' values;


   if( Wfc00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc00aa : "f10.2" : myid="i3" ; ll="i3" ; calling cb02aa ;")')cput-cpus,myid,ll
   endif
   
!latex \item \verb+cb02aa+ : computes Fourier harmonics of $p+B^2/2$ and spectral contraints on interfaces;
   if( ierror.eq.0 ) then ; CALL(cb02aa)
   endif

   if( Wfc00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc00aa : "f10.2" : myid="i3" ; ll="i3" ; called  cb02aa ;")')cput-cpus,myid,ll
   endif

    
   Bomn(ll,0:1,1:mn) = Bdmn(0:1,1:mn) ! Bdmn is defined in globals, allocated in al00aa, and computed in cb02aa;
   Iomn(ll,    2:mn) = Idmn(    2:mn) ! Idmn is defined in globals, allocated in al00aa, and computed in cb02aa;
   

   if( Wfc00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc00aa : "f10.2" : myid="i3" ; ll="i3" ; calling vo00aa ;")')cput-cpus,myid,ll
   endif

!latex \item \verb+vo00aa+ : calculate volume integral;
   CALL(vo00aa)

   if( Wfc00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc00aa : "f10.2" : myid="i3" ; ll="i3" ; called  vo00aa ;")')cput-cpus,myid,ll
   endif


   if( Wfc00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc00aa : "f10.2" : myid="i3" ; ll="i3" ; calling fu00aa ;")')cput-cpus,myid,ll
   endif

!latex \item \verb+fu00aa+ : calculate volume integrals of pressure, $B^2$ and ${\bf A}\cdot{\bf B}$.
   CALL(fu00aa)

   if( Wfc00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc00aa : "f10.2" : myid="i3" ; ll="i3" ; called  fu00aa ;")')cput-cpus,myid,ll
   endif


   if( Wfc00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc00aa : "f10.2" : myid="i3" ; ll="i3" ; calling ma00ac ;")')cput-cpus,myid,ll
   endif

!latex \item \verb+ma00ac+ : deallocates \verb+igss+, etc. ;
   CALL(ma00ac)

   if( Wfc00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc00aa : "f10.2" : myid="i3" ; ll="i3" ; called  ma00ac ;")')cput-cpus,myid,ll
   endif

   
  enddo ! end of do ll = 1 , Nvol loop;

!latex \end{enumerate}

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  if( Wfc00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc00aa : "f10.2" : myid="i3" ; calling bc00aa ;")')cput-cpus,myid
  endif

  do ll = 1 , Nvol ; CALL(bc00aa) !latex \item The routine \verb+bc00aa+ broadcasts data that needs to be shared.
  enddo

  if( Wfc00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc00aa : "f10.2" : myid="i3" ; called  bc00aa ;")')cput-cpus,myid
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex and the force vector is constructed.

  do ll = 1,Nvol-1
   df( (ll-1)*(mn+mn-1)   +1 : (ll-1)*(mn+mn-1)+mn      ) = Bomn(ll+1,0,1:mn) - Bomn(ll,1,1:mn) ! pressure imbalance;
   df( (ll-1)*(mn+mn-1)+mn+1 : (ll-1)*(mn+mn-1)+mn+mn-1 ) = Iomn(ll,2:mn)                       ! spectral deviation;
  enddo
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

! screen diagnostics; used elsewhere;

  forceerr = sqrt( sum(df*df) / lNgdof )
  
  do ll = 1 , Nvol-1
   mBomn(ll) = log10(max(sqrt(sum((Bomn(ll+1,0,1:mn)-Bomn(ll,1,1:mn))**2)/ mn   ),machprec)) ! screen diagnostics; used elsewhere;
   mIomn(ll) = log10(max(sqrt(sum((Iomn(ll  ,  2:mn)                )**2)/(mn-1)),machprec)) ! screen diagnostics; used elsewhere;
  enddo

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

! compute integrals

  lbeta(0) = sum( volume(0,1:Nvol)                         ) ! total             volume   ;
  lbeta(1) = sum( volume(0,1:Nvol) * pres(1:Nvol) * pscale ) ! total integral of pressure ;
  lbeta(2) = sum( volume(1,1:Nvol)                         ) ! total integral of B.B      ;
  lbeta(3) = sum( volume(2,1:Nvol) *   mu(1:Nvol)          ) ! total integral of A.B      ;

  beta = lbeta(1) / ( lbeta(1) + lbeta(2) )

  FF(0) = lbeta(1) + half * lbeta(2) - half * lbeta(3)

!latex \heading{theory}

!latex \item The energy functional, plus the angle spectral constraint, in each annulus is
!latex \be F_l =                 \int_{{\cal V}_l} \!\!\! dv \left( \frac{p}{\gamma-1}+\frac{B^2}{2} \right)
!latex         - \frac{\mu_l}{2} \int_{{\cal V}_l} \!\!\! dv \left(   {\bf A}\cdot{\bf B}            \right)
!latex         -       \nu_l     \int_{{\cal V}_l} \!\!\! dv       \, p^{1/\gamma}
!latex         +       \lambda_l \frac{1}{2} \sum_j (m_j^p+n_j^q)(R_j^2+Z_j^2).
!latex \ee
!latex The Lagrange multipliers, $\mu_l$ and $\nu_l$ are to be determined below.
!latex The factor $\lambda_l$ is to be decreased until the minimization of the spectral constraint (a purely numerical constraint)
!latex does not impact on force-balance (a physical constraint).

!latex \item The variation in $F_l$ due to variations in the pressure is 
!latex \be \delta F_l = \int_{{\cal V}_l} \!\!\! dv \, \delta p \left( \frac{1}{\gamma-1}-\frac{\nu_l p^{1/\gamma}}{\gamma p}\right)\ee
!latex Hereafter, we assume that $\nu_l p^{1/\gamma}=\gamma p/(\gamma-1)$, so that the pressure is constant in each volume.

!latex \item The Euler-Lagrange equations for the variations in the field and interface geometry are connected:
!latex if the interface geometry changes, the magnetic field must also change in order to ensure that the field remain tangential to the interfaces.
!latex On the interfaces we use $\delta {\bf A}=\delta \boldxi \times {\bf B}$ and derive
!latex \be \delta F_l = \int_{{\cal V}_l} \!\!\! dv \, \delta {\bf A} \cdot \left( \nabla \times {\bf B}-\mu_l{\bf B}\right)
!latex                - \int_{{\cal \partial V}_l} \!\!\! \delta \boldxi \cdot d{\bf S} \, \left(p+B^2/2\right)
!latex                + \lambda_l \sum_j (m_j^p+n_j^q)(R_j \delta R_j + Z_j \delta Z_j )
!latex \ee
!latex Hereafter, we assume that $\nabla \times {\bf B}=\mu_l{\bf B}$, and that $\mu_l$ is {\em initially} adjusted to satisfy the interface transform constraint.

!latex \item Position is ${\bf x}=R \, \hat r + Z \, \hat z$, where $\hat r = \cos \phi \, {\bf i} + \sin \phi \, {\bf j}$
!latex and $\hat \phi = -\sin \, \phi {\bf i} + \cos \phi \, {\bf j}$.
!latex The coordinate transformation is $R=R(\s,\t,\z)$, $\phi=-\z$, $Z=Z(\s,\t,\z)$.
!latex \item The area element is $ d{\bf S} \equiv {\bf e}_\t \times {\bf e}_\z d\t d\z =  [R Z_\t \hat r + ( Z_\t R_\z - R_\t Z_\z )\hat \phi - R R_\t \hat z ]d\t d\z$
!latex \item The variation in position is $\delta \boldxi = \delta R \, \hat r + \delta Z \, \hat z$,
!latex and so $\delta \boldxi \cdot d{\bf S} = R ( \delta R Z_\t - \delta Z R_\t ) d\t d\z$.

! FF(0) = - lbeta(1) + half * ( lbeta(2) - lbeta(3) ) ! total free energy functional;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!  FF(1:Ngdof) = zero ! initialize summation;
!  
!  do ll = 2,Nvol-1 ! loop over geometry degrees of freedom;
!   
!   do imn = 1 , mn
!    do jmn = 1 , mn
!     do kmn = 1 , mn
!      do lmn = 1 , mn
!       FF(idof) = FF(idof) - aijkl(imn,jmn,kmn,lmn) * ( Bomn(ll+1,0,jmn) - Bomn(ll,1,jmn) ) * Rbc(ll,kmn) * Zbs(ll,lmn)
!       FF(idof) = FF(idof) - aijkl(imn,jmn,kmn,lmn) * ( Bomn(ll+1,0,jmn) - Bomn(ll,1,jmn) ) * Rbc(ll,kmn) * Rbc(ll,lmn)
!      enddo
!     enddo
!    enddo
!   enddo
!   
!  enddo
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  RESETCPU

  RETURN(fc00aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
 end subroutine fc00aa
 
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
