!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Given vector position returns force.
!latex \item The force vector, ${\bf F}({\bf x})$, is a combination of the pressure-imbalance Fourier harmonics, $\left([[p+B^2/2]]\sqrt g^i\right)_{m,n}$, 
!latex where $i\equiv$\verb+Iusesg+ and the spectral condensation constraints, $\partial M/\partial \lambda_j$.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine fc01aa( lNgdof , xx , df , lderiv )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, half, one
  use numerical, only : machprec
  use fileunits, only : ounit
  use inputlist, only : Wfc01aa, ext, Nvol, Ntor, Lcurlerr, Lsetmupf, Hsetmupf, pres, pscale, mu, beta, forceerr, Nquad
  use cputiming
  use allglobal, only : numprocs, myid, cpus, &
                        Ngdof, &
                        ll, mn, im, in, &
                        Rbc,  Zbs, oRbc, oZbs, &
                        RZl, Bdmn, Idmn, Bomn, Iomn, dBd, dIdRZ, dfd, pHess, &
                        mBomn, mIomn, volume, FF, lextrap
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  LOCALS
  
  INTEGER, intent(in)  :: lNgdof       ! dimension;
  REAL,    intent(in)  :: xx(1:lNgdof) ! vector, representing degrees-of-freedom = internal geometry;
  REAL,    intent(out) :: df(1:lNgdof) ! force;
  LOGICAL, intent(in)  :: lderiv     ! indicates whether derivatives are to be calculated;
  
  LOGICAL              :: loriginal, lperturbed
  INTEGER              :: RZlmin, RZlmax, iRZl, iRZ, imn, jmn, idof, icon, imnc, ierror, liota
  REAL                 :: lbeta(0:3)

  BEGIN(fc01aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  FATALMESS(fc01aa,lNgdof.ne.Ngdof,array dimensions inconsistent)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item The vector, ${\bf x}$, represents the geometrical degrees of freedom of the internal interfaces.
!latex This vector is `unpacked' and the \verb+Rbc+, \verb+Zbs+ arrays are assigned.

  do ll = 1,Nvol-1 ; Rbc(ll,1:mn) = xx((ll-1)*(mn+mn-1)+1   :(ll-1)*(mn+mn-1)+mn     ) ! unpack coordinate degrees-of-freedom vector;
                     Zbs(ll,2:mn) = xx((ll-1)*(mn+mn-1)+mn+1:(ll-1)*(mn+mn-1)+mn+mn-1)
  enddo

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; calling ex00aa ;")')cput-cpus,myid
  endif

  CALL(ex00aa,(lextrap)) ! extrapolates coordinate harmonics to define innermost interface;

  if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; called  ex00aa ;")')cput-cpus,myid
  endif


  if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; calling ih00aa ;")')cput-cpus,myid
  endif

  CALL(ih00aa,(Nvol)) ! fits piecewise cubic interpolation to coordinate harmonics;

  if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; called  ih00aa ;")')cput-cpus,myid
  endif
  

  oRbc(0:Nvol-1,1:mn) = Rbc(0:Nvol-1,1:mn) ! save original geometry;
  oZbs(0:Nvol-1,1:mn) = Zbs(0:Nvol-1,1:mn) ! save original geometry;


!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item The following routines are called in parallel: \begin{enumerate}

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; looping over ll ;")')cput-cpus,myid
  endif

  do ll = 1 , Nvol ! construct Beltrami fields in parallel;
   
   if( myid.ne.modulo(ll-1,numprocs) ) cycle
   
   if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; ll="i3" ;")')cput-cpus,myid,ll
   endif
   
   if( lderiv ) then ; RZlmin = RZl(ll,0) ; RZlmax = RZl(ll,1)
   else              ; RZlmin =     ll    ; RZlmax =     ll 
   endif
   
   if( Nvol.le.1 ) RZlmax = RZlmin ! no internal interfaces; provide dummy values for RZlmin,RZlmax;
   
   if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; ll="i3" ; calling ma00ab ;")')cput-cpus,myid,ll
   endif

   CALL(ma00ab) !latex \item \verb+ma00ab+ : allocates sub-grid geometric arrays, \verb+igss+, \verb+igst+, etc. in each annulus;

   if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; ll="i3" ; called  ma00ab ;")')cput-cpus,myid,ll
   endif
   
   do iRZl = RZlmin , RZlmax ! loop over interfaces;
    do iRZ = 1,0,-1 ! loop over RZ; first Z then R;
     do imn = 1,mn ! loop over Fourier harmonics;
      
      jmn = iRZ*mn + imn - iRZ ; idof = ( iRZl - 2 ) * ( mn + mn - 1 ) + jmn ! label degree of freedom; only required if lderiv;
      
      if( iRZl.eq.RZlmin .and. iRZ.eq.1 .and. imn.eq.1 ) then ; loriginal = .true.  ; lperturbed = .not.loriginal
      else                                                    ; loriginal = .false. ; lperturbed = .not.loriginal
      endif
      
      if( .not.lderiv .and. lperturbed ) cycle ! nothing to be done;
      
      if( iRZl.gt.RZlmin .and. iRZ.eq.1 .and. imn.eq.1 ) cycle ! no dependence on Zbs_{0,0};
      
      Rbc(0:Nvol-1,1:mn) = oRbc(0:Nvol-1,1:mn) ! always reset geometry to original;
      Zbs(0:Nvol-1,1:mn) = oZbs(0:Nvol-1,1:mn)
      
!latex \item if \verb+lderiv=T+, the Fourier harmonics of the internal interfaces is displaced by an amount \verb+dfd+
!latex and the derivative matrix is constructed by finite-differences.

      if( lderiv .and. lperturbed ) then
       if( iRZ.eq.0 ) Rbc(iRZl,imn) = Rbc(iRZl,imn) + dfd ! forward differencing; recall that dfd is set in al00aa and is determined by input parameter ifinited;
       if( iRZ.eq.1 ) Zbs(iRZl,imn) = Zbs(iRZl,imn) + dfd
      endif
      
      if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; iRZl,iRZ,imn="3i6" ; calling ex00aa ;")')cput-cpus,myid,iRZl,iRZ,imn
      endif

      CALL(ex00aa,(lextrap)) !latex \item \verb+ex00aa+ : sets the position of the coordinate axis and innermost surface by extrapolation;
      
      if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; iRZl,iRZ,imn="3i6" ; called  ex00aa ;")')cput-cpus,myid,iRZl,iRZ,imn
      endif
      
      if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; iRZl,iRZ,imn="3i6" ; calling ih00aa ;")')cput-cpus,myid,iRZl,iRZ,imn
      endif

      CALL(ih00aa,(Nvol)) !latex \item \verb+ih00aa+ : constructs global coordinates by interpolating the interface Fourier harmonics;
      
      if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; iRZl,iRZ,imn="3i6" ; called  ih00aa ;")')cput-cpus,myid,iRZl,iRZ,imn
      endif
      
      if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; iRZl,iRZ,imn="3i6" ; calling ma00aa ;")')cput-cpus,myid,iRZl,iRZ,imn
      endif

      CALL(ma00aa,(Nquad)) !latex \item \verb+ma00aa+ : calls \verb+me00ab+ on sub-sub-grid to compute the geometric quantities, \verb+intgl+, on sub-grid;
      
      if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; iRZl,iRZ,imn="3i6" ; called  ma00aa ;")')cput-cpus,myid,iRZl,iRZ,imn
      endif
      
      if( loriginal ) then ; liota = Lsetmupf ! adjust helicity multiplier, mu, and poloidal flux, dpflux, to satisfy interface rotational transform constraint; see ma02aa;
      else                 ; liota = Hsetmupf ! adjust mu & dpflux during calculation of the derivative matrix;
      endif
      
     !if( ll.eq.1 ) then ! SKIP INNERMOST VOLUME;
     ! CALL(ma00ac)
     ! cycle ! not quite ready for calculation of Beltrami field in innermost volume;
     !endif
      
!latex \item \verb+ma02aa+ : iteratively adjusts the helicity and poloidal flux to match the interface constraints, where the Beltrami fields are computed by \verb+mp00aa+;
      
      if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; iRZl,iRZ,imn="3i6" ; calling ma02aa ;")')cput-cpus,myid,iRZl,iRZ,imn
      endif

      CALL(ma02aa,(liota,ierror)) 
      
      if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; iRZl,iRZ,imn="3i6" ; called  ma02aa ;")')cput-cpus,myid,iRZl,iRZ,imn
      endif
      
      Bdmn(0,1:mn) = zero ; Bdmn(1,1:mn) = one ; Idmn(1:mn) = one

!latex \item \verb+cb02aa+ : computes $p+B^2/2$ and spectral contraints, $\partial M / \partial \lambda_j$, on the interfaces, 
!latex where $M\equiv\sum_k(m_k^p+n_k^q)(R_k^2+Z_k^2)$ is the `spectral-width'
!latex and $\lambda_j$ is the freedom in the angle coordinate, $\theta = \bar \theta + \sum_j \lambda_j \sin(m_j \bar \theta-n_j \zeta)$.
      
      if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; iRZl,iRZ,imn="3i6" ; calling cb02aa ;")')cput-cpus,myid,iRZl,iRZ,imn
      endif

      if( ierror.eq.0 ) then ; CALL(cb02aa) 
      endif
            
      if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; iRZl,iRZ,imn="3i6" ; called  cb02aa ;")')cput-cpus,myid,iRZl,iRZ,imn
      endif

      if( loriginal ) then
       Bomn(ll,0:1,1:mn) = Bdmn(0:1,1:mn) ! Bdmn is defined in globals, allocated in al00aa, and computed in cb02aa;
       Iomn(ll,    2:mn) = Idmn(    2:mn) ! unperturbed geometry; construct function;
      endif
      
      if( lperturbed ) then ! assign derivative array;
       
       FATAL(fc01aa,.not.lderiv)
        
       dBd(ll,0)%mnllRZ(1:mn,iRZl,jmn) = ( Bdmn(0,1:mn) - Bomn(ll,0,1:mn) ) / dfd ! compute derivative; need to normalize; 
       dBd(ll,1)%mnllRZ(1:mn,iRZl,jmn) = ( Bdmn(1,1:mn) - Bomn(ll,1,1:mn) ) / dfd ! compute derivative; need to normalize; 
       
       if( iRZl.eq.ll ) dIdRZ(ll,2:mn,jmn) = ( Idmn(2:mn) - Iomn(ll,2:mn) ) / dfd ! spectral constraints are local to the surface;
!     !if( iRZl.eq.ll .and. ll.lt.Nvol ) dIdRZ(ll,2:mn,jmn) = ( Idmn(2:mn) - Iomn(ll,2:mn) ) / dfd ! spectral constraints are local to the surface;

      endif ! end of if( lperturbed ) then 
      
      if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; iRZl,iRZ,imn="3i6" ; calling vo00aa ;")')cput-cpus,myid,iRZl,iRZ,imn
      endif

      CALL(vo00aa) ! calculate volume integral;

      if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; iRZl,iRZ,imn="3i6" ; called  vo00aa ;")')cput-cpus,myid,iRZl,iRZ,imn
      endif

      if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; iRZl,iRZ,imn="3i6" ; calling fu00aa ;")')cput-cpus,myid,iRZl,iRZ,imn
      endif

      CALL(fu00aa) ! calculate volume integral A.B and B.B;

      if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; iRZl,iRZ,imn="3i6" ; called  fu00aa ;")')cput-cpus,myid,iRZl,iRZ,imn
      endif

     enddo ! end of do imn = 1,mn ! loop over Fourier harmonics;
       
    enddo ! end of do iRZ = 1,0,-1 ! loop over RZ harmonics;
   enddo ! end of do iRZl = RZlmin , RZlmax ! loop over interfaces;
    
! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! !

   Rbc(0:Nvol-1,1:mn) = oRbc(0:Nvol-1,1:mn) ! return to original geometry;
   Zbs(0:Nvol-1,1:mn) = oZbs(0:Nvol-1,1:mn) ! return to original geometry;
   
!   if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; calling ex00aa ;")')cput-cpus,myid
!   endif
!
!   CALL(ex00aa,(lextrap)) ! reset innermost interface geometry;
!
!   if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; called  ex00aa ;")')cput-cpus,myid
!   endif

   if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; calling ih00aa ;")')cput-cpus,myid
   endif

   CALL(ih00aa,(Nvol)) ! reset coordinate interpolation;

   if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; called  ih00aa ;")')cput-cpus,myid
   endif
   
! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! !

   if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; calling ma00ac ;")')cput-cpus,myid
   endif

   CALL(ma00ac) ! \item \item \verb+ma00ac+ deallocates sub-grid geometric arrays \verb+intgl+;
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  enddo ! end of do ll = 1 , Nvol loop; !latex \end{enumerate}
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; calling bc00aa ;")')cput-cpus,myid
  endif
  
!latex \item After broadcasting the required data via a call to \verb+bc00aa+,

  do ll = 1,Nvol ; CALL(bc00aa) ! loop over volumes; ll is global parameter; cannot parallelize this as data is to be broadcast;
  enddo
  
  if( Wfc01aa ) then ; cput = MPI_WTIME() ; write(ounit,'("fc01aa : "f10.2" : myid="i3" ; constructing force-vector ;")')cput-cpus,myid
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
!latex the force vector is constructed.

  do ll = 1,Nvol-1 ; df( (ll-1)*(mn+mn-1)   +1 : (ll-1)*(mn+mn-1)+mn      ) = Bomn(ll+1,0,1:mn) - Bomn(ll,1,1:mn) ! force imbalance;
                     df( (ll-1)*(mn+mn-1)+mn+1 : (ll-1)*(mn+mn-1)+mn+mn-1 ) = Iomn(ll,2:mn)                       ! spectral deviation;
  enddo
  
  forceerr = sqrt( sum(df*df) / lNgdof ) ! screen diagnostics; used elsewhere;

  do ll = 1,Nvol-1
   mBomn(ll) = log10(max(sqrt(sum((Bomn(ll+1,0,1:mn)-Bomn(ll,1,1:mn))**2)/ mn   ),machprec)) ! screen diagnostics; used elsewhere;
   mIomn(ll) = log10(max(sqrt(sum((Iomn(ll  ,  2:mn)                )**2)/(mn-1)),machprec)) ! screen diagnostics; used elsewhere;
  enddo

  lbeta(0) = sum( volume(0,2:Nvol)                         ) ! total             volume   ;
  lbeta(1) = sum( volume(0,2:Nvol) * pres(2:Nvol) * pscale ) ! total integral of pressure ;
  lbeta(2) = sum( volume(1,2:Nvol)                         ) ! total integral of B.B      ;
  lbeta(3) = sum( volume(2,2:Nvol) *   mu(2:Nvol)          ) ! total integral of A.B      ;

  beta = lbeta(1) / ( lbeta(1) + lbeta(2) )

  FF(0) = - lbeta(1) + half * ( lbeta(2) - lbeta(3) ) ! total free energy functional;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( lderiv ) then !latex \item If \verb+lderiv=T+ then the derivative matrix, \verb+pHess+ will be constructed. 
   
   FATAL(fc01aa,.not.allocated(pHess))
   
  !nnz=0 ; icon = 0 ; HH(1:nHnz)=zero ; pHess(1:lNgdof,1:lNgdof) = zero ! must first construct sparse Hessian;
   
   pHess(1:lNgdof,1:lNgdof) = zero ! Hessian;
   
   icon = 0
   
   do ll = 1,Nvol-1
    
    do imnc = 1,mn ; icon = icon+1
     
     do iRZl = minval( RZl(ll:ll+1,0) ) , maxval( RZl(ll:ll+1,1) )
      do iRZ = 0,1
       do imn = 1,mn ; jmn = iRZ*mn + imn - iRZ
        
        if( iRZ.eq.1 .and. imn.eq.1 ) cycle ! nothing depends on Zmn_{0,0}
        
        idof = ( iRZl - 2 ) * ( mn + mn - 1 ) + jmn ! label degree of freedom;
        
       !nnz = nnz + 1 ; hrow(nnz) = icon ; hcol(nnz) = idof
         
       !HH(nnz) = zero
         
       !if( iRZl.ge.RZl(ll  ,0) .and. iRZl.le.RZl(ll  ,1) ) HH(nnz) = HH(nnz) - dBd(ll  ,1)%mnllRZ(imnc,iRZl,jmn)
       !if( iRZl.ge.RZl(ll+1,0) .and. iRZl.le.RZl(ll+1,1) ) HH(nnz) = HH(nnz) + dBd(ll+1,0)%mnllRZ(imnc,iRZl,jmn)
 
        if( iRZl.ge.RZl(ll  ,0) .and. iRZl.le.RZl(ll  ,1) ) pHess(icon,idof) = pHess(icon,idof) - dBd(ll  ,1)%mnllRZ(imnc,iRZl,jmn)
        if( iRZl.ge.RZl(ll+1,0) .and. iRZl.le.RZl(ll+1,1) ) pHess(icon,idof) = pHess(icon,idof) + dBd(ll+1,0)%mnllRZ(imnc,iRZl,jmn)
          
       !pHess(icon,idof) = HH(nnz)
          
       enddo ! end of do imn=1,mn;
      enddo ! end of do iRZ=0,1;
     enddo ! end of do iRZl=
     
    enddo ! end of do imnc=1,mn
    
    do imnc = 2,mn ; icon = icon+1
     
     do iRZ = 0,1
      do imn = 1,mn ; jmn = iRZ*mn + imn - iRZ
       
       if( iRZ.eq.1 .and. imn.eq.1 ) cycle ! nothing depends on Zmn_{0,0}
       
       idof = (  ll  - 1 ) * ( mn + mn - 1 ) + jmn ! label degree of freedom;
  
      !nnz = nnz+1 ; hrow(nnz) = icon ; hcol(nnz)=idof
   
      !HH(nnz) = dIdRZ(ll,imnc,jmn)
   
      !pHess(icon,idof) = HH(nnz)
 
       pHess(icon,idof) = dIdRZ(ll,imnc,jmn)
       
      enddo ! end of do imn=1,mn
     enddo ! end of do iRZ=0,1
     
    enddo ! end of do imnc=2,mn;
    
   enddo ! end of do ll=2,Nvol-1;
   
  !FATAL(he00aa,nnz.ne.nHnz)
    
  endif ! end of if( lderiv ) then;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RESETCPU

  RETURN(fc01aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine fc01aa
 
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
