!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Locates invariant irrational surfaces of a Hamiltonian flow
!latex using method described in [Hudson, 2004] \cite{Hudson_04}.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

module curveflow
  INTEGER :: j,k,Nk 
  REAL, allocatable :: sap(:,:),tap(:,:) ! full invariant surface on regular (alpha,zeta) grid;
end module curveflow

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item subroutine \verb+invarianterr+

!latex \begin{enumerate}

! the format of this routine is consistent with that required by E04GYF, which is used below in ir00aa;
subroutine invarianterr(Nc,Nf,xc,err,de,ljc,iuser,user)
  
!latex \item This computes the `function' and the `function-derivative'; this routine is called by \verb+ir00aa+ below;

  use constants!, only : 
  use fileunits!, only : ounit
  use inputlist, only : Wir00aa, odetol, irrMpol, irrNtor
  use cputiming, only : Tir00aa
  use allglobal, only : cpus, irrsurf, pi2nfp, ll, myid
  
  use curveflow
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS
  
  INTEGER :: Nc,Nf,ljc,iuser(1)
  REAL :: xc(0:Nf-1),err(0:Nc-1),de(0:ljc-1,0:Nf-1),user(*)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  INTEGER :: m,Node,ifail,Npts
  REAL :: zst,zend,tol,Tst(6),TWk(20*6)!,bfend
  REAL, allocatable :: ts(:),si(:),ti(:),tsb(:),sb(:),tb(:),st(:),tt(:)
  REAL, allocatable :: dst(:,:,:) ! tangent map;
  
  CHARACTER :: R
  EXTERNAL :: bf00aa, curveflowout, bf00aa_end

  BEGIN(ir00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( (Nf-1)/2 .ne. irrMpol ) stop "invarianterr : Fourier resolution problem;"

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  Npts=Nf ; Npts=Nf+1 ! +1 is to keep Npts even; easier to avoid prime factor problem appearing below;
   
  Nk=max(2*irrNtor,1) ! discrete resolution in toroidal direction determined by Fourier resolution;
 
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
 
! straight-field line angle;
  ALLOCATE(ts,(0:Npts-1))
  ts(0:Npts-1) = (/(j,j=0,Npts-1)/)*pi2/Npts ! straight-field line angle;
  
! rigid-shifted straight-field line angle;
  ALLOCATE(tsb,(0:Npts-1))
  tsb = ts + pi2nfp * irrsurf%iota ! rigid-shifted straight-field line angle;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
! original s, t;
  ALLOCATE(si,(0:Npts-1)) 
  ALLOCATE(ti,(0:Npts-1))
  
!  mapped s, t;
  ALLOCATE(st,(0:Npts-1))
  ALLOCATE(tt,(0:Npts-1))
  
! tangent map;
  ALLOCATE(dst,(0:Npts-1,2,2))
  
! rigid shift s, t;
  ALLOCATE(sb,(0:Npts-1)) 
  ALLOCATE(tb,(0:Npts-1))
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item A trial curve, represented by a discrete set of points $(s_i,\theta_i)$,
!latex \begin{equation}\begin{array}{ccccc}      s_i&=&                      & &\sum_{m=0}^{M}      s_m \cos(m \alpha_i ), \\
!latex                                      \theta_i&=&  \alpha              &+&\sum_{m=1}^{M} \theta_m \sin(m \alpha_i ),
!latex \end{array}\end{equation} 
!latex where $\alpha$ is a straight field line angle, and $\alpha_i$ are regularly spaced, and $M$=\verb!irrMpol!,

  si(0:Npts-1) = xc(0) ; ti(0:Npts-1) = ts(0:Npts-1) ! initial (s,t) ;
  do m = 1,irrMpol
   si(0:Npts-1) = si(0:Npts-1) + xc(        m) * cos(m*ts(0:Npts-1))
   ti(0:Npts-1) = ti(0:Npts-1) + xc(irrMpol+m) * sin(m*ts(0:Npts-1))
  enddo
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex is mapped under the field line flow $(s,\theta)\mapsto(\tilde s,\tilde \theta)$, 
!latex with the field (and tangent map) given by \verb+bf00ac+,
  
  do j = 0,Npts-1 ! j is a counter provided to curveflowout below;

   k=0 ! k is a counter provided to curveflowout below;

   Tst(1:6) = (/ si(j) , ti(j) , one , zero , zero , one /) ; zst=zero ; zend=pi2nfp ; Node=6 ; tol=odetol ; R='D' ; TWk=zero

   ifail=0
   call D02BJF( zst , zend , Node , Tst , bf00aa , tol , R , curveflowout , bf00aa_end  , TWk , ifail ) ! field-line Hamiltonian;
  !call D02BJF( zst , zend , Node , Tst , ph00aa , tol , R , curveflowout , bfend , TWk , ifail ) ! pressure-jump Hamiltonian;

   st(j)=Tst(1) ; tt(j)=Tst(2) ; dst(j,1,1)=Tst(3) ; dst(j,1,2)=Tst(4) ; dst(j,2,1)=Tst(5) ; dst(j,2,2)=Tst(6)

  enddo
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
 
!latex and compared to the rigidly shifted curve, 
!latex \begin{equation}\begin{array}{ccccc} \bar      s&=&             & &\sum      s_m \cos[m \bar \alpha ], \\
!latex                                      \bar \theta&=& \bar \alpha &+&\sum \theta_m \sin[m \bar \alpha ],
!latex \end{array}\end{equation}
!latex where $\bar \alpha = \alpha+2\pi\iotabar$.
  
  sb(0:Npts-1) = xc(0) ; tb(0:Npts-1) = tsb(0:Npts-1) ! rigid shift (s,t) ;
  do m = 1,irrMpol
   sb(0:Npts-1) = sb(0:Npts-1) + xc(        m) * cos(m*tsb(0:Npts-1))
   tb(0:Npts-1) = tb(0:Npts-1) + xc(irrMpol+m) * sin(m*tsb(0:Npts-1))
  enddo
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item The `function' is the error vector is given ${\bf e}=(\bar s - \tilde s, \bar \theta - \tilde \theta)^T$.
  
  err(0:Npts-1)=sb(0:Npts-1)-st(0:Npts-1) ; err(Npts:2*Npts-1)=tb(0:Npts-1)-tt(0:Npts-1) ! error;
  
  irrsurf%err=sqrt( sum(err(0:2*Npts-1)**2)/(2*Npts) )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
 
!latex The correction to the harmonics, $s_m$ and $\theta_m$, are obtained via the derivative matrix:
!latex \begin{equation} \begin{array}{ccccc}

!latex \partial(\bar      s - \tilde      s)/\partial      s_m &=
!latex & \cos(m \bar \alpha) &-& \cos(m \alpha) \partial \tilde      s/\partial      s, \\ 
!latex \partial(\bar      s - \tilde      s)/\partial \theta_m &=
!latex &                     &-& \sin(m \alpha) \partial \tilde      s/\partial \theta, \\ 
!latex \partial(\bar \theta - \tilde \theta)/\partial      s_m &=
!latex &                     &-& \cos(m \alpha) \partial \tilde \theta/\partial      s, \\ 
!latex \partial(\bar \theta - \tilde \theta)/\partial \theta_m &=
!latex & \sin(m \bar \alpha) &-& \sin(m \alpha) \partial \tilde \theta/\partial \theta,
!latex \end{array} \end{equation}
!latex where the $\partial \tilde x / \partial y$ are elements of the tangent map.

  de(0:2*Npts-1,0:2*irrMpol)=zero ! error derivative matrix;
  
   de(   0:  Npts-1,0)         = cos(0*tsb) - dst(0:Npts-1,1,1)!* cos(0*ts)
   de(Npts:2*Npts-1,0)         =            - dst(0:Npts-1,2,1)!* cos(0*ts)
  
  do m=1,irrMpol
   de(   0:  Npts-1,m        ) = cos(m*tsb) - dst(0:Npts-1,1,1) * cos(m*ts)
   de(   0:  Npts-1,m+irrMpol) =            - dst(0:Npts-1,1,2) * sin(m*ts)
   de(Npts:2*Npts-1,m        ) =            - dst(0:Npts-1,2,1) * cos(m*ts)
   de(Npts:2*Npts-1,m+irrMpol) = sin(m*tsb) - dst(0:Npts-1,2,2) * sin(m*ts)
  enddo
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  DEALLOCATE(ts)
  DEALLOCATE(tsb)
  
  DEALLOCATE(si)
  DEALLOCATE(ti)
  
  DEALLOCATE(st)
  DEALLOCATE(tt)
  
  DEALLOCATE(dst)
  
  DEALLOCATE(sb)
  DEALLOCATE(tb)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  iuser(1)=iuser(1)+1

  return
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine invarianterr

!latex \end{enumerate}

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item subroutine \verb+ir00aa+

!latex \item Execution flow is controlled by \verb+Hamiltonian+, which is passed through \verb+allglobal+: 
!latex for field-line analysis, \verb+Hamiltonian=0+;  for pressure-jump analysis, \verb+Hamiltonian=1+.

!latex \begin{enumerate}

subroutine ir00aa
  
  use constants!, only : 
  use fileunits!, only : 
  use inputlist, only : Wir00aa, irrMpol,irrNtor,odetol,irrsvdcut,Mirrits,irrtol,ext,Nfp,p1,q1,p2,q2!,pqirr
  use cputiming
  use allglobal, only : ll,irrsurf,cpus,pi2nfp,mn,im,in,irrmn,irrim,irrin,Bzeta,myid

  use curveflow

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS 

  INTEGER :: jk,m,n,Nc,Nf,ifail,its,Npts,imn,iuser(1),ljc!,Lw
  REAL :: zeta,ths,cs(2),arg,gtt,gtp,gpp,osqrtNptsNk,user(1)!,sumsq,cyl_err
  REAL :: dRsp(0:3),dZsp(0:3)
  REAL, allocatable :: xc(:),err(:),de(:,:),dm(:),wsvd(:)!,MWk(:)
  REAL, allocatable :: sjk(:),tjk(:),Rjk(:,:),Zjk(:,:),BR(:),Bp(:),BZ(:),Blow(:,:),irrBtmn(:),irrBpmn(:)
  REAL, allocatable :: E(:),trm(:),trn(:),Fwk(:) ! do not confuse these with the global arrays;

  REAL :: stz(6),Bstz(6) ! really just position, but bf00aa also returns tangent map so input array is length 6;
  REAL :: RpZ(3),dR(0:3),dZ(0:3),lsg,lgij(1:6) 

  CHARACTER :: isr,suff*19

  LOGICAL :: Lsvd=.true. !, Lnag=.false. ! it seems that homespun svd works better than nag;
 !external :: invarianterr ! only required if Lnag=.true.

  INTEGER :: icount,irank
  INTEGER, allocatable :: rank(:)
 !REAL :: Btperr,Btp(2)

  BEGIN(ir00aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  if( Mirrits.le.0 ) then
   goto 9999
  !RETURN(ir00aa)
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  Nf=2*irrMpol+1 ! degrees of freedom in stellarator symmetric invariant curve;
  Npts=Nf ; Npts=Nf+1 ! ; Npts=2*Npts ! +1 is to keep Npts even; easier to avoid prime factor problem appearing below;
 !Npts = (irrMpol+1) is sufficient for surfaces; insufficient for subsequent Fourier decomposition; need aliasing check;
  
  Nc=2*Npts ; Nk=max(2*irrNtor,1) ! discrete resolution determined by Fourier resolution;
  osqrtNptsNk = one/sqrt(one*Npts*Nk)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

! full surface s, t on regular (alpha,zeta) grid; including periodic curve;
  ALLOCATE(sap,(0:Npts,0:Nk-1)) 
  ALLOCATE(tap,(0:Npts,0:Nk-1))

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

! independent variable, error vector and derivative matrix; provided by tagent map;
  ALLOCATE(xc,(0:2*irrMpol))
  ALLOCATE(err,(0:2*Npts-1))
  ALLOCATE(de,(0:2*Npts-1,0:2*irrMpol))

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if(Lsvd) then
   
! singular values and correction;
   ALLOCATE(wsvd,(1:Nf)) 
   ALLOCATE(dm,(0:2*irrMpol))
   
   iuser(1)=0 ; ifail=1
   
!latex \item Given an initial guess for the irrational invariant {\em curve}, 
   
   do its=1,Mirrits ! until error is satisfactory;
    
    xc(        0:  irrMpol) = irrsurf%sm(0:irrMpol)
    xc(irrMpol+1:2*irrMpol) = irrsurf%tm(1:irrMpol)
    
!latex the routine \verb+invarianterr+ is called iteratively (maximum iterations given by \verb+Mirrits+)
    
    ljc=Nc
    CALL(invarianterr,( Nc , Nf , xc , err , de , ljc , iuser , user ))
    
!latex until the root mean square error is less than \verb+irrtol+.
    
    irrsurf%err=sqrt(sum(err(0:2*Npts-1)**2)/(2*Npts))
    
!    if( irrsurf%err.le.irrtol ) then ; ifail=0 ; exit ! always perform max. iterations allowed, as set by Mirrits;
!    endif
    
!latex Corrections are calculated by singular value decomposition inversion (see \verb+singvalues+) of the derivative matrix.
    
    call singvalues( Nc , Nf , de( 0:2*Npts-1 , 0:2*irrMpol ) , err(0:2*Npts-1) , dm(0:2*irrMpol) , irrsvdcut , wsvd )
    
    irrsurf%sm(0:irrMpol) = irrsurf%sm(0:irrMpol) - dm(        0:  irrMpol) ! update s-Fourier harmonics of curve;
    irrsurf%tm(1:irrMpol) = irrsurf%tm(1:irrMpol) - dm(irrMpol+1:2*irrMpol) ! update t-Fourier harmonics of curve;
    
    cput = MPI_WTIME()
    write(ounit,2001)cput-cpus,myid,ll,irrsurf%iota,irrMpol,irrsurf%sm(0),irrsurf%err,iuser(1),ifail,cput-cpuo
    cpuo=cput

   enddo ! end of do its=1,Mirrits; until error is satisfactory;
   
   irrsurf%its=iuser(1)
   
   cput = MPI_WTIME()
  !write(ounit,2001)cput-cpus,irrsurf%iota,irrMpol,irrsurf%sm(0),irrsurf%err,iuser(1),ifail,cput-cpuo ; cpuo=cput

   DEALLOCATE(wsvd)
   DEALLOCATE(dm)
   
  endif

2001 format("ir00aa : "f10.2" : myid="i3" ; ll="i3" ; iota="f15.12" ; M="i4" ; sm(0)="f9.6" ; ":"rms="es9.2&
   " ; its="i2" ; ":"ifail="i2" ; time="f10.2"s ;")

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item (There is an option to solve sum-of-squares using NAG routine \verb+E04GYF+.
!latex This routine does not perform as well as the SVD routine above. 
!latex I think this is probably due to integration errors, which may confuse the NAG routine.)
!  if(Lnag) then 
!
!   xc(                 0:  irrMpol) = irrsurf%sm(0:irrMpol)
!   xc(irrMpol+1:2*irrMpol) = irrsurf%tm(1:irrMpol)
!   
!   Lw = 8*Nf + 2*Nf*Nf + 2*Nc*Nf + 3*Nc
!   
!   ALL!OCATE(MWk,(Lw))
!   MWk=zero
!   iuser=0 ; user=zero
!   
!   ifail=1 ; call E04GYF( Nc, Nf, invarianterr, xc, sumsq, MWk, Lw, iuser, user, ifail ) ; irrsurf%err=sqrt(sumsq/(2*Npts))
!   
!   DEALL!OCATE(MWk)
!   
!   irrsurf%sm(0:irrMpol)=xc(          0 :  irrMpol)
!   irrsurf%tm(1:irrMpol)=xc(irrMpol+1:2*irrMpol)
!
!   irrsurf%its=iuser(1)
!
!   cpu(1)=MPI_WTIME() 
!   if(WF(5)) write(ounit,2001)cpu(1)-cpustart,Hamiltonian,iota,irrMpol,irrsurf%sm(0),irrsurf%err,iuser(1),ifail ; cpuo=cput
!
!  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  DEALLOCATE(xc)
  DEALLOCATE(err)
  DEALLOCATE(de)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item If the rms error tolerance is greater than \verb!irrtol(2)!,
!latex a Fourier representation of the invariant {\em surface} will {\bf not} be constructed.

  cput = MPI_WTIME()
  write(ounit,'("ir00aa : "f10.2" : irrational rms error ="es9.2" ; irrtol ="es9.2" ;")')cput-cpus,irrsurf%err,irrtol ; cpuo=cput

  if( irrsurf%err.gt.irrtol ) then
   DEALLOCATE(sap) ; DEALLOCATE(tap)
   goto 9999
  !RETURN(ir00aa) ! need to check below if file output will be corrupted by this abrupt return;
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item The invariant {\em surface} is obtained by allowing the invariant {\em curve} to `flow' along the field. 
!latex Note: the surface thus constructed is initially obtained on a regular $(\alpha,\zeta)$ grid, 
!latex where $\alpha$ labels field lines.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  sap(Npts,0:Nk-1)=sap(0,0:Nk-1)     ! full surface on complete domain; regular alpha grid; destroyed by Fourier decomposition;
  tap(Npts,0:Nk-1)=tap(0,0:Nk-1)+pi2 ! full surface on complete domain; regular alpha grid; destroyed by Fourier decomposition;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

! full surface s, t on regular (theta_s,zeta) grid;
  ALLOCATE(sjk,(Npts*Nk))
  ALLOCATE(tjk,(Npts*Nk))

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex To obtain a regular grid in $(\theta_s,\zeta)$, where $\theta_s$ is the straight-field line angle consistent with $\zeta$,
!latex it is required to interpolate.
!latex It is (probably) more accurate to interpolate to get the background coordinates $(s,\theta,\zeta)$, 
!latex which probably vary slightly, 
!latex and then perform the (exact) transformation to cylindrical coordinates, $(R,\phi,Z)$.
  
  do k=0,Nk-1 ; zeta=k*pi2nfp/Nk

   N=Npts ; ifail=1 ; call slowft(sap(0:Npts-1,k),N,ifail)
   sap(0:Npts-1,k) = sap(0:Npts-1,k)/sqrt(one*Npts) ; sap(1:Npts-1,k)=sap(1:Npts-1,k)*two

   tap(0:Npts,k) = tap(0:Npts,k) - ( (/(j,j=0,Npts)/)*pi2/Npts + irrsurf%iota*zeta ) ! lambda;

   N=Npts ; ifail=1 ; call slowft(tap(0:Npts-1,k),N,ifail)
   tap(0:Npts-1,k) = tap(0:Npts-1,k)/sqrt(one*Npts) ; tap(1:Npts-1,k)=tap(1:Npts-1,k)*two

   do j=0,Npts-1 ; jk=1+j+Npts*k ; ths=j*pi2/Npts ! ; alpha = ths - iota*zeta;

    sjk(jk)=sap(0,k) ; tjk(jk)=ths+tap(0,k)

    do m=1,irrMpol ; cs = (/ cos( m*(ths-irrsurf%iota*zeta) ) , sin( m*(ths-irrsurf%iota*zeta) ) /) ! grid is irregular;
     sjk(jk) = sjk(jk) + sap(m,k)*cs(1) - sap(Npts-m,k)*cs(2)
     tjk(jk) = tjk(jk) + tap(m,k)*cs(1) - tap(Npts-m,k)*cs(2)
    enddo

   enddo
  enddo
  
   cput = MPI_WTIME()
!   if(WF(3)) write(ounit,2002)cput-cpus,cput-cpuo ; cpuo=cput
!2002 format("ir00aa : "f10.2" : WF(3) ;":" interpolated to regular straight-field-line grid ; time="f10.2"s ;")

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!  do k=0,Nk-1
!   write(ounit,'("sjk="99f7.3)')(sjk(jk),jk=1+Npts*k,1+Npts-1+Npts*k)
!  enddo
!  pause
!
!  do k=0,Nk-1
!   write(ounit,'("tjk="99f7.3)')(tjk(jk),jk=1+Npts*k,1+Npts-1+Npts*k)
!  enddo
!  pause

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

! full surface R, Z on regular (theta_s,zeta) grid;
  ALLOCATE(Rjk,(2,Npts*Nk)) 
  ALLOCATE(Zjk,(2,Npts*Nk))

! covariant cylindrical B^R, B^phi, B^Z on regular (theta_s,zeta) grid;
  ALLOCATE(BR,(Npts*Nk)) 
  ALLOCATE(Bp,(Npts*Nk))
  ALLOCATE(BZ,(Npts*Nk))
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  Rjk(1,:)=zero ; Zjk(1,:)=zero

  ADDCPU(ir00aa)

  do j=0,Npts-1
   do k=0,Nk-1 ; jk = 1+j+Npts*k ; zeta = k*pi2nfp/Nk

    stz(1:3)=(/ sjk(jk) , tjk(jk) , zeta /)

    CALL(co00aa,( stz(1:3) , RpZ(1:3) , dR(0:3) , dZ(0:3) , lsg , lgij ))
    
    Rjk(1,jk)=RpZ(1) ; Zjk(1,jk)=RpZ(3)

! contravariant field in background coordinates; NOTE: missing Jacobian factors;
    stz(3:6)=(/ one , zero , zero , one /)
    CALL(bf00aa,( zeta , stz(1:6) , Bstz(1:6) ) )

!latex \item The contravariant field is transformed to cylindrical coordinates via
!latex \be 
!latex \left ( \begin{array}{c} B^R \\ B^\phi \\ B^Z \end{array} \right)  = 
!latex \left ( \begin{array}{ccc}   R_s  &   R_\theta  &   R_\zeta  \\ 
!latex                               0   &      0      &     -1     \\ 
!latex                              Z_s  &   Z_\theta  &   Z_\zeta  \end{array} \right)
!latex \left ( \begin{array}{c} B^s \\ B^\theta \\ B^\zeta \end{array} \right).
!latex \ee
!latex Recall that $B_\phi = R^2 B^\phi$.
    
    Bstz(1:3) = (/ Bstz(1) , Bstz(2) , one /) * (Bzeta/lsg) ! Bzeta=B^\zeta is returned through global memory;
    
    BR(jk) = dR(1)*Bstz(1) + dR(2)*Bstz(2) + dR(3)*Bstz(3)
    Bp(jk) =                                     - Bstz(3) * Rjk(1,jk)**2 ! convert contravariant to covariant;
    BZ(jk) = dZ(1)*Bstz(1) + dZ(2)*Bstz(2) + dZ(3)*Bstz(3)
    
   enddo
  enddo

  RESETCPU

  cput = MPI_WTIME()

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  DEALLOCATE(sjk)
  DEALLOCATE(tjk)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

! these are local trigonometric arrays; do not confuse with global arrays;
  ALLOCATE(E,(Npts*Nk))     
  ALLOCATE(trm,(2*Npts))
  ALLOCATE(trn,(2*Nk))
  ALLOCATE(Fwk,(2*Npts*Nk))

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item The surface is Fourier decomposed in the straight field line angle, 
!latex with harmonics idenified by \verb+irrim(1:irrmn)+ and \verb+irrin(1:irrmn)+, which are constructed in \verb+readin+. 
!latex \item The surface geometry harmonics are stored in the global arrays \verb+irrsurf%Rbc(:)+, \verb+irrsurf%Zbs(:)+.

  Rjk(2,:)=Rjk(1,:) ; Zjk(2,:)=Zjk(1,:) ! save original data for Fourier reconstruction check;

  E=zero ; isr='I' ; ifail=0 ; call C06FUF( Npts , Nk , Rjk(2,1:Npts*Nk) , E , isr , trm , trn , Fwk , ifail )
  Rjk(2,:) = Rjk(2,:)*osqrtNptsNk ; Rjk(2,2:) = Rjk(2,2:)*two
  E=zero ; isr='S' ; ifail=0 ; call C06FUF( Npts , Nk , Zjk(2,1:Npts*Nk) , E , isr , trm , trn , Fwk , ifail )
  E = E*osqrtNptsNk ; E(2:)= E(2:)*two ! Fourier decomposition;
  do imn=1,irrmn ; j=irrim(imn) ; k=modulo(Nk-irrin(imn),Nk)
   irrsurf%Rbc(imn)= Rjk(2,1+j+k*Npts) ; irrsurf%Zbs(imn)=-E(1+j+k*Npts)
  enddo

   cput = MPI_WTIME()
!   if(WF(3)) write(ounit,2004)cput-cpus,cput-cpuo ; cpuo=cput
!2004 format("ir00aa : "f10.2" : WF(3) ;":" Fourier decomposed cylindrical harmonics of irrational surface ; time="f10.2"s ;")

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

! this section is just for debugging; reconstructs cylindrical surface from Fourier representation; 

!  Rjk(2,:)=zero ; Zjk(2,:)=zero ; cyl_err=zero ! set summation quantities to zero;
!
!  do j=0,Npts-1 
!   do k=0,Nk-1 ; jk=1+j+Npts*k
!    
!! straight line coordinates of original cylindrical (R,Z) data set;
!    do imn=1,irrmn ; arg = irrim(imn) * (j*pi2/Npts) - irrin(imn) * (k*pi2nfp/Nk) ; cs=(/ cos(arg),sin(arg) /) 
!     Rjk(2,jk) = Rjk(2,jk) + irrsurf%Rbc(imn)*cs(1) ! Fourier reconstruction;
!     Zjk(2,jk) = Zjk(2,jk) + irrsurf%Zbs(imn)*cs(2) ! Fourier reconstruction;
!    enddo
!
!   !write(ounit,'("ir00aa : "f10.2" : WF(?) : R="3f15.10" , Z="3f15.10" ;")')&
!   !zero,Rjk(1:2,jk),Rjk(2,jk)-Rjk(1,jk),Zjk(1:2,jk),Zjk(2,jk)-Zjk(1,jk);pause
!
!    cyl_err = cyl_err + ( Rjk(2,jk)-Rjk(1,jk) )**2 + ( Zjk(2,jk)-Zjk(1,jk) )**2
!
!   enddo
!  enddo
!
   cput = MPI_WTIME()
!   if(WF(3)) write(ounit,3005)cput-cpus,cyl_err,cput-cpuo ; cpuo=cput ! ; pause
!3005 format("ir00aa : "f10.2" : WF(3) ;":" cylindrical coordinate reconstruction error = "es10.2" ; time="f10.2"s ;")

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  DEALLOCATE(Rjk)
  DEALLOCATE(Zjk)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  DEALLOCATE(sap)
  DEALLOCATE(tap)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

! tangential contravariant B^s, and covariant components B_{\theta_s}, B_\zeta, where s is irrational surface label;
  ALLOCATE(Blow,(1:5,Npts*Nk)) 
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!latex \item The vector transformation to straight-field-line coordinates should exploit fast Fourier transforms.

  do j=0,Npts-1 ; ths=j*pi2/Npts ! construct straight-field-line coordinate derivatives, perform vector transformation;
   do k=0,Nk-1 ; zeta=k*pi2nfp/Nk ; jk=1+j+Npts*k
  
    dRsp(0:3)=zero ; dZsp(0:3)=zero
  
    do imn = 1 , irrmn ; arg = irrim(imn)*ths - irrin(imn)*zeta ; cs=(/ cos(arg) , sin(arg) /)
     
     dRsp(0) = dRsp(0) + irrsurf%Rbc(imn)*cs(1)
    !dRsp(1) = ?? no radial coordinate is constructed for discrete irrational surfaces;
     dRsp(2) = dRsp(2) + irrsurf%Rbc(imn)*cs(2)*(-irrim(imn) )
     dRsp(3) = dRsp(3) + irrsurf%Rbc(imn)*cs(2)*( irrin(imn) )
    !dZsp(0) = dZsp(0) + irrsurf%Zbs(imn)*cs(2) ; not required;            
    !dZsp(1) = ?? no radial coordinate is constructed for discrete irrational surfaces;
     dZsp(2) = dZsp(2) + irrsurf%Zbs(imn)*cs(1)*( irrim(imn) )     
     dZsp(3) = dZsp(3) + irrsurf%Zbs(imn)*cs(1)*(-irrin(imn) )
     
    enddo

!latex \item The covariant field is transformed 
!latex \be 
!latex \left ( \begin{array}{c} B_s \\ B_\theta \\ B_\zeta \end{array} \right)  = 
!latex \left ( \begin{array}{ccc}   R_s          &   0  &   Z_s           \\ 
!latex                              R_{\theta_s} &   0  &   Z_{\theta_s}  \\ 
!latex                              R_\zeta      &  -1  &   Z_\zeta       \end{array} \right)
!latex \left ( \begin{array}{c} B_R \\ B_\phi \\ B_Z \end{array} \right).
!latex \ee

    Blow(2,jk) = dRsp(2) * BR(jk)          + dZsp(2) * BZ(jk)
    Blow(3,jk) = dRsp(3) * BR(jk) - Bp(jk) + dZsp(3) * BZ(jk)
    
! the following is for checking the field normal to the invariant surface is zero, and the transform is correct, . . .;
    
!latex \item The normal field is 
!latex \be {\bf B} \cdot {\bf e}_{\theta_s} \times {\bf e}_{\zeta} = 
!latex Z_{\theta_s} R B_R + ( Z_{\theta_s} R_\zeta - R_{\theta_s}Z_\zeta ) B_\phi R^{-1} - R_{\theta_s} R B_z.
!latex \ee
    
! B^s; omitting jacobian factor; debugging;
    Blow(1,jk) = dZsp(2)*dRsp(0)*BR(jk) + ( dZsp(2)*dRsp(3)-dRsp(2)*dZsp(3) ) * Bp(jk) / dRsp(0) - dRsp(2)*dRsp(0)*BZ(jk) 
    
!latex \item The transform is given 
!latex \be \iotabar = \frac{B^{\theta_s}}{B^\zeta} = \frac{ B_{\theta_s} g_{\zeta\zeta} - B_\zeta g_{\theta_s\zeta}}
!latex                     { -B_{\theta_s} g_{\theta_s\zeta} + B_\zeta g_{\zeta\zeta}}.
!latex \ee
    
    gtt = dRsp(2)*dRsp(2) + dZsp(2)*dZsp(2)
    gtp = dRsp(2)*dRsp(3) + dZsp(2)*dZsp(3)
    gpp = dRsp(3)*dRsp(3) + dZsp(3)*dZsp(3) + dRsp(0)**2
    
    Blow(4,jk) =  Blow(2,jk)*gpp - Blow(3,jk)*gtp ! B^{\theta_s} (well, not really -- missing some normalization factors);
    Blow(5,jk) = -Blow(2,jk)*gtp + Blow(3,jk)*gtt ! B^{\zeta}    (well, not really -- missing same normalization factors);
    
   enddo
  enddo

   cput = MPI_WTIME()
   write(ounit,2005)cput-cpus,sqrt((/sum(Blow(1,:)**2),sum((irrsurf%iota-Blow(4,:)/Blow(5,:))**2)/)/(Npts*Nk)),cput-cpuo
   cpuo=cput
2005 format("ir00aa : "f10.2" :":" covariant transformation ; normal field error ="es10.2" ; transform error ="es10.2&
   " ; need to implement fast Fourier transforms ; time="f10.2"s ;")
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  DEALLOCATE(BR)
  DEALLOCATE(Bp)
  DEALLOCATE(BZ)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

! Fourier harmonics of irrational surfaces : tangential covariant field;
  ALLOCATE(irrBtmn,(1:irrmn))
  ALLOCATE(irrBpmn,(1:irrmn))

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  Blow(4,:)=Blow(2,:) ; Blow(5,:)=Blow(3,:) ! save for Fourier reconstruction check;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  E=zero ; isr='I' ; ifail=0 ; call C06FUF(Npts,Nk,Blow(4,1:Npts*Nk),E,isr,trm,trn,Fwk,ifail)
  Blow(4,:)=Blow(4,:)*osqrtNptsNk ; Blow(4,2:)=Blow(4,2:)*two
  E=zero ; isr='S' ; ifail=0 ; call C06FUF(Npts,Nk,Blow(5,1:Npts*Nk),E,isr,trm,trn,Fwk,ifail)
  Blow(5,:)=Blow(5,:)*osqrtNptsNk ; Blow(5,2:)=Blow(5,2:)*two
  do imn=1,irrmn ; j=irrim(imn) ; k=modulo(Nk-irrin(imn),Nk) ; irrBtmn(imn)= Blow(4,1+j+k*Npts) ; irrBpmn(imn)= Blow(5,1+j+k*Npts)
  enddo

   cput = MPI_WTIME()
!   if(WF(3)) write(ounit,2007)cput-cpus,cput-cpuo ; cpuo=cput
!2007 format("ir00aa : "f10.2" : WF(3) ;":" Fourier decomposed covariant field on irrational surface; time="f10.2"s ;")

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!  Btperr=zero ! Fourier reconstruction check only; comment out after verification;
!  do j=0,Npts-1 ; ths=j*pi2/Npts ! reconstruct Fourier decomposition;
!   do k=0,Nk-1 ; zeta=k*pi2nfp/Nk ; jk=1+j+Npts*k ; Btp(1:2)=zero ! initialize summation variables;
!    do imn=1,irrmn ; Btp(1:2) = Btp(1:2) + (/irrBtmn(imn),irrBpmn(imn)/) * cos( irrim(imn)*ths-irrin(imn)*zeta )
!    enddo
!    Btperr = Btperr + ( Blow(2,jk)-Btp(1) )**2 + ( Blow(3,jk)-Btp(2) )**2
!   enddo
!  enddo
!
!   cput = MPI_WTIME()
!   if(WF(3)) write(ounit,3008)cput-cpus,sqrt(Btperr/(Npts*Nk)),cput-cpuo ; cpuo=cput
!3008 format("ir00aa : "f10.2" : WF(3) ;":" covariant Fourier reconstruction error = "es10.2" ; time="f10.2"s ;")

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  DEALLOCATE(Blow)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

! these are local trigonometric arrays; do not confuse with global arrays;
  DEALLOCATE(E)
  DEALLOCATE(trm)
  DEALLOCATE(trn)
  DEALLOCATE(Fwk)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item The surface potential $f$, where $B_{\theta_s}=\partial_{\theta_s} f$, $B_{\zeta}=\partial_{\zeta} f$, 
!latex is calculated and written to \verb!ext.irr.**.pj!. 
!latex % in a format suitable for \verb!/u/shudson/SPEC/miscellaneous/preshamil!.;

!latex \item The surface potential $f$, where $B_{\theta_s}=\partial_{\theta_s} f$ and $B_{\zeta}=\partial_{\zeta} f$, 
!latex is given by the 
!latex harmonics which are stored in \verb+irrsurf%fmn(:)+.
!latex The secular terms in the surface potential, ie. $I$ and $G$, are by the `constant' harmonics, 
!latex and saved in \verb+irrsurf%Itor+, \verb+irrsurf%Gpol+.

  irrsurf%Itor=zero ; irrsurf%Gpol=zero ; irrsurf%fmn(:)=zero

  do imn=1,irrmn ! integrate to get surface potential from covariant components of field;
   if(     irrim(imn).eq.0 .and. irrin(imn).eq.0 ) then
    irrsurf%Itor=irrBtmn(imn) ; irrsurf%Gpol=irrBpmn(imn)
   elseif( irrim(imn).eq.0 .and. irrin(imn).ne.0 ) then
    irrsurf%fmn(imn) =                             - irrBpmn(imn) / irrin(imn)
   elseif( irrim(imn).ne.0 .and. irrin(imn).eq.0 ) then
    irrsurf%fmn(imn) =   irrBtmn(imn) / irrim(imn)   
   elseif( irrim(imn).ne.0 .and. irrin(imn).ne.0 ) then
    irrsurf%fmn(imn) = ( irrBtmn(imn) / irrim(imn) - irrBpmn(imn) / irrin(imn) ) * half ! should be equal ?;
   endif
  enddo

  ALLOCATE(rank,(irrmn))
  ifail=0 ; call M01DAF(abs(irrsurf%fmn),1,irrmn,'D',rank,ifail) ! rank the surface potential harmonics;
  
  icount=0
  do irank=1,irrmn ! loop over all the ranks;
   do imn=1,irrmn ! loop over all the harmonics;
    if( rank(imn).eq.irank ) then
     if( irrim(imn).ne.0 .and. irrin(imn).ne.0 ) then ; icount=icount+1 ! can only compare the m.ne.0 & n.ne.0 harmonics;
      write(ounit,2008) irrim(imn) , irrin(imn) , irrBtmn(imn) / irrim(imn) , - irrBpmn(imn) / irrin(imn) , &
    ( irrBtmn(imn) / irrim(imn) + irrBpmn(imn) / irrin(imn) )
     endif
    endif
   enddo
   if(icount.ge.5) exit ! only show the top harmonics (to reduce screen clutter);
  enddo
  
  DEALLOCATE(rank)
  
2008 format("ir00aa : "10x" : surface potential mode (m,n)=("i3" ,"i3" ) ; ( Btmn/m , -Bpmn/n )=("es11.3" ,"es11.3&
   " ) ;":" err="es9.1" ; ") ! should compare . . . .

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  DEALLOCATE(irrBtmn)
  DEALLOCATE(irrBpmn)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  write(suff,'(i2.2":"i2.2":"i2.2":"i2.2":"i3.3":"i3.3)')p1(ll),q1(ll),p2(ll),q2(ll),irrMpol,irrNtor
  
  cput = MPI_WTIME() ; write(ounit,'("ir00aa : "f10.2" : WF(3) ;":" writing ext."a19".pjh ; time="f10.2"s ;")')cput-cpus,suff,cput-cpuo ; cpuo=cput
  
  open(irrationunit,file=trim(ext)//"."//suff//".pjh",status="unknown")
  write(irrationunit,'(3i6)')irrMpol,irrNtor,Nfp
  write(irrationunit,'(2es23.15,4i6,es23.15)')irrsurf%Itor,irrsurf%Gpol,p1(ll),q1(ll),p2(ll),q2(ll),irrsurf%iota!=(p1*goldmean+p2)/(q1*goldmean+q2)
  do imn=1,irrmn ; write(irrationunit,'(2i6,3es23.15)')irrim(imn),irrin(imn),irrsurf%fmn(imn),irrsurf%Rbc(imn),irrsurf%Zbs(imn)
  enddo
  close(irrationunit)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  RETURN(ir00aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

end subroutine ir00aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \end{enumerate}

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine curveflowout(iz,Tst) ! used for mapping invariant curve through to invariant surface;
  use allglobal,only : pi2nfp
  use curveflow
  implicit none

  REAL, intent(inout) :: iz ! toroidal (integration) variable;
  REAL, intent(in) :: Tst(6)

  sap(j,k) = Tst(1) ; tap(j,k) = Tst(2) ! fill in surface array;

  k = k+1 ; iz = k * pi2nfp / Nk

  return
end subroutine curveflowout

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!



