!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item Calls NAG routine to find zero of force-balance using user supplied derivative matrix.

!latex \item This routine is called from \verb+spec+, and only if \verb+Lc05pdf.eq.true+.
!latex The NAG routine \verb+C05PDF+ is used to find a zero of the force-balance vector, as computed by \verb+fc00aa+.
!latex The iterations are terminated if $|{\bf x}-{\bf x}_o|<$\verb+forcetol+.

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine jk02aa( lNgdof , xtry , ic05pdf )
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, one, ten
  use numerical, only : 
  use fileunits, only : ounit, iterationunit, derivativeunit
  use inputlist, only : ext, Nvol, Mpol, Ntor, forcetol, factor, LuseDM, DMext, beta, forceerr, Wjk02aa
  use cputiming, only : Tjk02aa
  use allglobal, only : writin, myid, cpus, mn, Bomn, Iomn, pHess, Ngdof, &
                        mn, im, in, Rbc, Zbs, & ! required for unpacking geometry vector in preparation for call to writin;
                        mBomn, mIomn ! these are returned by fc01aa;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  LOCALS

  INTEGER, intent(in)    :: lNgdof
  REAL   , intent(inout) :: xtry(1:lNgdof)
  INTEGER, intent(out)   :: ic05pdf

  LOGICAL                :: lderiv, exist, LcomputeDM
  INTEGER                :: irevcm, mode, ldfjac, LR, its(1:2), ll, idof, jdof, ijdof, oNgdof, oNvol, oMpol, oNtor
  REAL                   :: xtol, lfactor, mdf, cpul

  REAL                   :: soln(1:lNgdof)

  REAL                   :: fvec(1:lNgdof), fjac(1:lNgdof,1:lNgdof), DIAG(1:lNgdof), RR(1:lNgdof*(lNgdof+1)/2), QTF(1:lNgdof), WK(1:lNgdof,1:4)
  
  BEGIN(jk02aa)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  FATAL(jk02aa,lNgdof.ne.Ngdof)

  cpul = cpuo ; ic05pdf = 1
  
  xtol = forcetol ! set accuracy requirement;
  
  ldfjac = lNgdof ; LR = lNgdof * (lNgdof+1) / 2 ! local variables, array dimensions;

  mode = 0 ! if mode=2, multiplicative scale factors need to be provided in diag; if mode=0, scaling factors computed internally;
  
  lfactor = factor ! used to determine initial step bound;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
!  if( Wjk02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk02aa : "f10.2" : myid="i3" ; allocating fvec(1:lNgdof) ; lNgdof="i9" ;")')cput-cpus,myid,lNgdof
!  endif
!
!  RALLOCATE(fvec,(1:lNgdof))
!
!  if( Wjk02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk02aa : "f10.2" : myid="i3" ; allocated  fvec(1:lNgdof) ; lNgdof="i9" ;")')cput-cpus,myid,lNgdof
!  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  its(1:2) = 0 ! counter;
  
  lderiv = .false. ! initially, will call fc01aa to calculate the force-imbalance; the Jacobian is not yet required;

  if( Wjk02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk02aa : "f10.2" : myid="i3" ; calling fc01aa ; with lderiv=.false. ;")')cput-cpus,myid
  endif
  
  ADDCPU(jk02aa)

  CALL(fc01aa,(lNgdof,xtry(1:lNgdof),fvec(1:lNgdof),lderiv))
  
  RESETCPU

  if( Wjk02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk02aa : "f10.2" : myid="i3" ; called  fc01aa ;")')cput-cpus,myid
  endif

  if( myid.eq.0 ) then
   cput = MPI_WTIME()
   mdf = sqrt( sum(fvec**2) / lNgdof )
   write(ounit,3011)cput-cpus,its(1:2),beta,forceerr,mdf,cput-cpul,(mBomn(ll),mIomn(ll),ll=1,Nvol-1)
   cpul = cput
   CALL(writin) ! write restart file; save geometry to ext.end; Rbc, Zbs consistent with xtry;
  endif
    
  if( forceerr.lt.forcetol*ten ) then ! force-balance is satisfied;
  !DEALLOCATE(fvec)
   ic05pdf = 0 
   goto 9999
!   if( Wjk02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk02aa : "f10.2" : myid="i3" ; finish ;")')cput-cpus,myid
!   endif
!   RETURN(jk02aa)
  endif
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

 !RALLOCATE(fjac,(1:ldfjac,1:lNgdof))
  
 !lderiv = .true. ! On entry to C05PDF, the "error" vector and Jacobian matrix are required; well, that's what the documentation says, but it seems to make no difference;
 !CALL(fc01aa,(lNgdof,xtry(1:lNgdof),fvec(1:lNgdof),lderiv)) ! compute force-balance "error" vector;

 !fjac(1:LDfjac,1:lNgdof) = pHess(1:lNgdof,1:lNgdof) ! should really include array dimensions here;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!  RALLOCATE(DIAG,(1:lNgdof))   ! scaling factors; only required on input if mode.eq.2;
!  RALLOCATE(RR,(1:LR))       ! upper triangular matrix in QR factorization of final approximate Jacobian;
!  RALLOCATE(QTF,(1:lNgdof))    ! on final exit, the vector Q^T f;
!  RALLOCATE(WK,(1:lNgdof,1:4)) ! workspace array;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  irevcm = 0 ; ic05pdf = 1 ! required for initial entry; herefater unchanged by user;
  
  if( myid.eq.0 ) then
   open(iterationunit,file="."//trim(ext)//".fits",status="unknown",iostat=ios)
   if( ios.ne.0 ) write(ounit,'("jk02aa : "f10.2" : error opening .ext.fits file;")')cput-cpus
  endif
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  if( Wjk02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk02aa : "f10.2" : myid="i3" ; entering reverse communication loop ;")')cput-cpus,myid
  endif

  do ! C05PDF uses reverse communication so is placed inside infinite loop;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   call C05PDF( irevcm , lNgdof , xtry , fvec , fjac , ldfjac , xtol , DIAG , mode , lfactor , RR , LR , QTF , WK(1:lNgdof,1:4) , ic05pdf )
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   select case(irevcm)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   case(0) ! final exit; C05PDF has terminated;

    mdf = sqrt( sum(fvec**2) / lNgdof )
    
    if( myid.eq.0 ) then
     cput = MPI_WTIME()
     select case(ic05pdf)
     case(0) ; write(ounit,'("jk02aa : "f10.2" ; success        ; ic05pdf="i2" ; mdf="es10.2" ; evaluations / derivatives ="2i4" ;")') cput-cpus,ic05pdf,mdf,its
     case(1) ; write(ounit,'("jk02aa : "f10.2" ; input error    ; ic05pdf="i2" ; mdf="es10.2" ; evaluations / derivatives ="2i4" ;")') cput-cpus,ic05pdf,mdf,its
     case(2) ; write(ounit,'("jk02aa : "f10.2" ; irevcm error   ; ic05pdf="i2" ; mdf="es10.2" ; evaluations / derivatives ="2i4" ;")') cput-cpus,ic05pdf,mdf,its
     case(3) ; write(ounit,'("jk02aa : "f10.2" ; xtol too small ; ic05pdf="i2" ; mdf="es10.2" ; evaluations / derivatives ="2i4" ;")') cput-cpus,ic05pdf,mdf,its
     case(4) ; write(ounit,'("jk02aa : "f10.2" ; bad progress   ; ic05pdf="i2" ; mdf="es10.2" ; evaluations / derivatives ="2i4" ;")') cput-cpus,ic05pdf,mdf,its
     case(5) ; write(ounit,'("jk02aa : "f10.2" ; bad progress   ; ic05pdf="i2" ; mdf="es10.2" ; evaluations / derivatives ="2i4" ;")') cput-cpus,ic05pdf,mdf,its
     case default
      FATALMESS(jk02aa,.true.,illegal ic05pdf : C05PDF error)
     end select
    endif
    
    if( LuseDM .and. myid.eq.0 ) then ! will save derivative matrix for future use;
     
     pHess(1:lNgdof,1:lNgdof) = zero
     ijdof = 0
     do idof = 1,lNgdof
      do jdof = idof , lNgdof ; ijdof = ijdof + 1 ; pHess(idof,jdof) = RR(ijdof) ! un-pack R matrix;
      enddo
     enddo
     fjac = matmul( fjac(1:lNgdof,1:lNgdof) , pHess(1:lNgdof,1:lNgdof) ) ! reconstruct derivative matrix = Q R;
     
     open(derivativeunit,file=".DM."//DMext,status="replace",form="unformatted",iostat=ios)
     if( ios.ne.0 ) write(ounit,'("jk02aa : "f10.2" : error opening (saving) derivative matrix file;")')cput-cpus

     if( ios.eq.0 ) then
      write(derivativeunit,iostat=ios)lNgdof,Nvol,Mpol,Ntor
      if( ios.ne.0 ) write(ounit,'("jk02aa : "f10.2" : error writing lNgdof,Nvol,Mpol,Ntor to derivative matrix file;")')cput-cpus
      write(derivativeunit,iostat=ios)fjac(1:lNgdof,1:lNgdof)
      if( ios.ne.0 ) write(ounit,'("jk02aa : "f10.2" : error writing fjac to derivative matrix file;")')cput-cpus
     endif
     
     close(derivativeunit,iostat=ios)
     if( ios.ne.0 ) write(ounit,'("jk02aa : "f10.2" : error closing (saving) derivative matrix file;")')cput-cpus

    endif ! end of if( myid.eq.0 ) then loop;

    exit ! escape from infinite do loop;

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   case(1) ! indicates start of new iteration; no action is required; xtry and fvec available for printing; fvec must not be changed;
    
    do ll = 2,Nvol-1 ; Rbc(ll,1:mn) = xtry((ll-2)*(mn+mn-1)+1   :(ll-2)*(mn+mn-1)+mn     ) ! unpack coordinate degrees-of-freedom vector;
                       Zbs(ll,2:mn) = xtry((ll-2)*(mn+mn-1)+mn+1:(ll-2)*(mn+mn-1)+mn+mn-1)
    enddo
    
    if( myid.eq.0 ) then
     cput = MPI_WTIME()
     mdf = sqrt( sum(fvec**2) / lNgdof )
     write(ounit,3011)cput-cpus,its(1:2),beta,forceerr,mdf,cput-cpul,(mBomn(ll),mIomn(ll),ll=1,Nvol-1)
     cpul=cput
     CALL(writin) ! write restart file; save geometry to ext.end; Rbc, Zbs consistent with xtry;
     write(iterationunit,'(2i9,2es23.15)',iostat=ios)its(1:2),forceerr,mdf ; call flush(iterationunit)
     if( ios.ne.0 ) write(ounit,'("jk02aa : "f10.2" : error writing its(1:2),forceerr,mdf to .ext.fits;")')cput-cpus
    endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   case(2) ! before re-entry to C05PDF, fvec must contain the function values;
    
    its(1) = its(1) + 1
    
    lderiv = .false. ! derivatives (computed in fc01aa) are not required;

    ADDCPU(jk02aa)

    if( Wjk02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk02aa : "f10.2" : myid="i3" ; calling fc01aa ; with lderiv=.false. ;")')cput-cpus,myid
    endif

    CALL(fc01aa,(lNgdof,xtry(1:lNgdof),fvec(1:lNgdof),lderiv)) !latex \item The force-balance `error' vector and its derivative are computed by \verb+fc01aa+.

    if( Wjk02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk02aa : "f10.2" : myid="i3" ; called  fc01aa ;")')cput-cpus,myid
    endif

    RESETCPU

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   case(3) ! before re-entry to C05PDF, fjac must contain the derivatives;
    
    its(2) = its(2) + 1 ; LcomputeDM = .true. ! derivative matrix evaluation counter;
    
    if( LuseDM .and. its(2).eq.1 ) then ! this is the first iteration;
     
     if( myid.eq.0 ) then ! reading files is not parallelizable;
      
      inquire(file=".DM."//DMext,exist=exist) ! the derivative matrix from any previous succesful call to jk02aa is saved;
      
      if( exist ) then !latex \item If \verb+LuseDM=T+, then at the first iteration the derivative matrix will be read from \verb+.derivative+.     

       open(derivativeunit,file=".DM."//DMext,status="old",form="unformatted",iostat=ios)
       if( ios.ne.0 ) write(ounit,'("jk02aa : "f10.2" : error opening (reading) derivative matrix file;")')cput-cpus

       read(derivativeunit,iostat=ios)oNgdof,oNvol,oMpol,oNtor
       if( ios.ne.0 .or. oNgdof.ne.lNgdof .or. oNvol.ne.Nvol .or. oMpol.ne.Mpol .or. oNtor.ne.Ntor ) then ; LcomputeDM = .true.  ! an error occurred reading .derivative;
       else                                                                                               ; LcomputeDM = .false.
       endif
       if( .not.LcomputeDM ) read(derivativeunit,iostat=ios) pHess(1:lNgdof,1:lNgdof)
       if( ios.ne.0 ) write(ounit,'("jk02aa : "f10.2" : error reading pHess from derivative matrix file;")')cput-cpus

       close(derivativeunit,iostat=ios)
       if( ios.ne.0 ) write(ounit,'("jk02aa : "f10.2" : error closing (reading) derivative matrix file;")')cput-cpus
       if( ios.ne.0 ) LcomputeDM = .true.

      endif ! end of if( exist );

      cput = MPI_WTIME() ; write(ounit,'("jk02aa : "f10.2" : derivative matrix read from file = "L2" ;")')cput-cpus,.not.LcomputeDM
     !its(2) = its(2) - 1

     endif ! end of if( myid.eq.0 );
     
     LlBCAST(LcomputeDM,1,0)
     RlBCAST(pHess,lNgdof*lNgdof,0)
     
    endif ! end of if( LuseDM .and. its(2).eq.1 ) then;
    
    if( LcomputeDM ) then ! usually the derivative matrix needs to be computed;
     
     ADDCPU(jk02aa)

     if( Wjk02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk02aa : "f10.2" : myid="i3" ; calling fc01aa ; with lderiv=.true.  ;")')cput-cpus,myid
     endif
     
     lderiv = .true. ; CALL(fc01aa,(lNgdof,xtry(1:lNgdof),soln(1:lNgdof),lderiv)) ! derivatives (computed in fc01aa) are required;

     if( Wjk02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk02aa : "f10.2" : myid="i3" ; called  fc01aa ;")')cput-cpus,myid
     endif
     
     RESETCPU
     
    endif
    
    fjac(1:ldfjac,1:lNgdof) = pHess(1:lNgdof,1:lNgdof)
    
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

   case default
    
    FATALMESS(jk02aa,.true.,illegal irevcm : C05PDF error)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
    
   end select ! end of select case(irevcm);

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  enddo ! end of infinite reverse-communication do loop;

  if( Wjk02aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jk02aa : "f10.2" : myid="i3" ; exit reverse communication do-loop ;")')cput-cpus,myid
  endif
  
  if( myid.eq.0 ) then
   close(iterationunit,iostat=ios)
   if( ios.ne.0 ) write(ounit,'("jk02aa : "f10.2" : error closing .ext.fits;")')cput-cpus
  endif

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

 !DEALLOCATE(fjac)
  
 !DEALLOCATE(DIAG) ; DEALLOCATE(RR) ; DEALLOCATE(QTF) ; DEALLOCATE(WK)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

 !DEALLOCATE(fvec)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  RETURN(jk02aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

3011 format("jk02aa : "f10.2" : "i5,i4" ; beta="f7.4" ; |f|="2es14.7" ; time="f8.1"s ; |BB|,|I|="999(f6.2,f5.1)" ;")

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

end subroutine jk02aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
