!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-

!latex \item Computes current, measures error in Beltrami equation.

!latex \item The magnetic field is given as the curl of the vector potential,
!latex \be {\bf B} = \frac{\partial_\t A_\z - \partial_\z A_\t}{\sqrt g}{\bf e}_\s
!latex             - \frac{                   \partial_\s A_\z}{\sqrt g}{\bf e}_\t
!latex             + \frac{\partial_\s A_\t                   }{\sqrt g}{\bf e}_\z
!latex             = B^\s {\bf e}_\s + B^\t {\bf e}_\t + B^\z {\bf e}_\z.
!latex \ee

!latex To take the curl of this, it is convenient to `lower' the metric elements
!latex \be B_\s&=&B^\s g_{\s\s}+B^\t g_{\s\t}+B^\z g_{\s\z},\\
!latex     B_\t&=&B^\s g_{\s\t}+B^\t g_{\s\t}+B^\z g_{\s\t},\\
!latex     B_\z&=&B^\s g_{\s\z}+B^\t g_{\s\z}+B^\z g_{\s\z}.
!latex \ee

!latex The curl is then 
!latex \be \nabla \times {\bf B} = \frac{\partial_\t B_\z - \partial_\z B_\t}{\sqrt g}{\bf e}_\s
!latex                           + \frac{\partial_\z B_\s - \partial_\s B_\z}{\sqrt g}{\bf e}_\t
!latex                           + \frac{\partial_\s B_\t - \partial_\t B_\s}{\sqrt g}{\bf e}_\z.
!latex \ee

!latex \item The interpolated vector potential harmonics and their derivatives \verb+Atmn(0:2,1:mn)+ and \verb+Azmn(0:2,1:mn)+,
!latex are calculated in \verb+sa00aa+.
!latex The derivatives of the metric elements \verb+gij(1:6,0:3,1:Ntz)+ and Jacobian \verb+sg(0:3,1:Ntz)+
!latex are calculated in \verb+co01aa+.

!latex \heading{volume integral}
!latex \item Define 
!latex \be ||\left( \nabla \times {\bf B}-\mu {\bf B}\right)\cdot\nabla \s || & \equiv & 
!latex   \int ds \ooint \sqrt g \left| \left( \nabla \times {\bf B}-\mu {\bf B}\right)\cdot\nabla \s \right|
!latex = \int ds \ooint \left[(\partial_\t B_\z - \partial_\z B_\t) - \mu (\partial_\t A_\z - \partial_\z A_\t)\right], \nonumber \\
!latex     ||\left( \nabla \times {\bf B}-\mu {\bf B}\right)\cdot\nabla \t || & \equiv & \int ds \ooint \sqrt g \left( \nabla \times {\bf B}-\mu {\bf B}\right)\cdot\nabla \t
!latex =\int ds \ooint \left[(\partial_\z B_\s - \partial_\s B_\z) - \mu (\;\;\;\;\;\;\;\; - \partial_\s A_\z)\right], \nonumber \\
!latex     ||\left( \nabla \times {\bf B}-\mu {\bf B}\right)\cdot\nabla \z || & \equiv & \int ds \ooint \sqrt g \left( \nabla \times {\bf B}-\mu {\bf B}\right)\cdot\nabla \s
!latex =\int ds \ooint \left[(\partial_\s B_\t - \partial_\t B_\s) - \mu (\partial_\s A_\t   \;\;\;\;\;\;\;\;)\right], \nonumber
!latex \ee

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

subroutine jo00aa(Ntz)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  use constants, only : zero, half
  use fileunits, only : ounit
  use inputlist, only : Wjo00aa, Ni, mu
  use cputiming
  use allglobal, only : myid, cpus, ll, iss, mn, im, in, &
                        trigwk, trigm, trign, isr, Nt, Nz, efmn, ofmn, cfmn, sfmn, &
                        sg, gij, Rij, Zij, Atmn, Azmn, ijreal, jireal, soNtz
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  LOCALS

  INTEGER, intent(in) :: Ntz ! this is really global, but is included in argument list to remove allocations;

  INTEGER             :: ii, ij, ik, il, Lcurv, ig, lNi, Nen = 2**2, ifail
  REAL                :: lss, ssmin, ssmax, dBu(1:3,0:3,1:Ntz), dBs(1:3,0:3,1:Ntz), jerror(1:3), jdenom(1:3) ! error in Fourier space;

  BEGIN(jo00aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  lNi = Ni(ll) * Nen ! slightly enhanced resolution;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  dBu(1:3,0:3,1:Ntz) = zero ; dBs(1:3,0:3,1:Ntz) = zero ! contravariant and covariant components of magnetic field;
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  jerror(1:3) = zero ; jdenom(1:3) = zero ! measure of error, (curl B - mu B) . \grad u_i
  
  ssmin = max( iss(1) * half**5 , iss(ll-1) ) ; ssmax = iss(ll)

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
  do ii = 1 , lNi-1 ! keep away from boundary as finite-differences used in co01aa;
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item The error is computed in real space on a grid.

   lss = ssmin + ii * ( ssmax-ssmin ) / lNi
   
   ADDCPU(jo00aa)

   if( Wjo00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jo00aa : "f10.2" : myid="i3" ; calling sa00aa ;")')cput-cpus,myid
   endif

   CALL(sa00aa,(lss)) ! returns interpolated Atmn(0:2,1:mn), Azmn(0:2,1:mn) ;

   if( Wjo00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jo00aa : "f10.2" : myid="i3" ; called  sa00aa ;")')cput-cpus,myid
   endif

   
   if( Wjo00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jo00aa : "f10.2" : myid="i3" ; calling co01aa ;")')cput-cpus,myid
   endif

   Lcurv = 2  ! returns interpolated Rij(0:9,1:Ntz), Zij(0:9,1:Ntz), sg(0:3,1:Ntz), gij(1:6,0:3,1:Ntz)
   CALL(co01aa(lss,Lcurv))

   if( Wjo00aa ) then ; cput = MPI_WTIME() ; write(ounit,'("jo00aa : "f10.2" : myid="i3" ; called  co01aa ;")')cput-cpus,myid
   endif

 
   RESETCPU

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   ofmn(1:mn) = -          im(1:mn)*Azmn(0,1:mn) -          in(1:mn)*Atmn(0,1:mn) ! harmonics of B^s;
   sfmn(1:mn) = -          im(1:mn)*Azmn(1,1:mn) -          in(1:mn)*Atmn(1,1:mn) ! harmonics of B^s; radial derivative;

   efmn(1:mn)=zero ; cfmn(1:mn)=zero
   call invfft( mn , im , in , efmn(1:mn) , ofmn(1:mn) , cfmn(1:mn) , sfmn(1:mn) , Nt , Nz , dBu(1,0,:) , dBu(1,1,:) , isr , trigm , trign , trigwk )

   efmn(1:mn) = - im(1:mn)*im(1:mn)*Azmn(0,1:mn) - im(1:mn)*in(1:mn)*Atmn(0,1:mn) ! harmonics of B^s; poloidal derivative;
   cfmn(1:mn) = + in(1:mn)*im(1:mn)*Azmn(0,1:mn) + in(1:mn)*in(1:mn)*Atmn(0,1:mn) ! harmonics of B^s; toroidal derivative;

   ofmn(1:mn)=zero ; sfmn(1:mn)=zero
   call invfft( mn , im , in , efmn(1:mn) , ofmn(1:mn) , cfmn(1:mn) , sfmn(1:mn) , Nt , Nz , dBu(1,2,:) , dBu(1,3,:) , isr , trigm , trign , trigwk )
   
! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! !

   efmn(1:mn) =                                  -                   Azmn(1,1:mn) ! harmonics of B^t;
   cfmn(1:mn) =                                  -                   Azmn(2,1:mn) ! harmonics of B^t; radial derivative;

   ofmn(1:mn)=zero ; sfmn(1:mn)=zero
   call invfft( mn , im , in , efmn(1:mn) , ofmn(1:mn) , cfmn(1:mn) , sfmn(1:mn) , Nt , Nz , dBu(2,0,:) , dBu(2,1,:) , isr , trigm , trign , trigwk )

   ofmn(1:mn) =                                  +          im(1:mn)*Azmn(1,1:mn) ! harmonics of B^t; poloidal derivative;
   sfmn(1:mn) =                                  -          in(1:mn)*Azmn(1,1:mn) ! harmonics of B^t; toroidal derivative;

   efmn(1:mn)=zero ; cfmn(1:mn)=zero
   call invfft( mn , im , in , efmn(1:mn) , ofmn(1:mn) , cfmn(1:mn) , sfmn(1:mn) , Nt , Nz , dBu(2,2,:) , dBu(2,3,:) , isr , trigm , trign , trigwk )
   
! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! ! !

   efmn(1:mn) = +                   Atmn(1,1:mn)                                  ! harmonics of B^z; 
   cfmn(1:mn) = +                   Atmn(2,1:mn)                                  ! harmonics of B^z; radial derivative;

   ofmn(1:mn)=zero ; sfmn(1:mn)=zero
   call invfft( mn , im , in , efmn(1:mn) , ofmn(1:mn) , cfmn(1:mn) , sfmn(1:mn) , Nt , Nz , dBu(3,0,:) , dBu(3,1,:) , isr , trigm , trign , trigwk )

   ofmn(1:mn) = -          im(1:mn)*Atmn(1,1:mn)                                  ! harmonics of B^z; poloidal derivative;
   sfmn(1:mn) = +          in(1:mn)*Atmn(1,1:mn)                                  ! harmonics of B^z; toroidal derivative;

   efmn(1:mn)=zero ; cfmn(1:mn)=zero
   call invfft( mn , im , in , efmn(1:mn) , ofmn(1:mn) , cfmn(1:mn) , sfmn(1:mn) , Nt , Nz , dBu(3,2,:) , dBu(3,3,:) , isr , trigm , trign , trigwk )
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

!latex \item To calculate $\nabla \times {\bf B}$ use the expression
!latex \be \partial_k B_i = \sum_j \left(
!latex   \frac{\partial_k B^j}{\sqrt g}g_{ij} 
!latex - \frac{B^j}{\sqrt g}\frac{\partial_k \sqrt g}{\sqrt g}g_{ij} 
!latex + \frac{B^j}{\sqrt g}\partial_k g_{ij} 
!latex \right).
!latex \ee
   
   dBs(1:3,1:3,1:Ntz) = zero ! initialize summation;
   
   do ik = 1,3
    do il = 1,3
     
     do ij = 1,3

      call gi00aa(il,ij,ig) ! ig is intent out; see numrec.h;
      
      dBs(il,ik,:)  =  dBs(il,ik,:) + ( ( dBu(ij,ik,:) - dBu(ij,0,:) * sg(ik,:) / sg(0,:) ) * gij(ig,0,:) + dBu(ij,0,:) * gij(ig,ik,:) ) / sg(0,:)
      
     enddo

    enddo
   enddo
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   ijreal = dBs(3,2,:) - dBs(2,3,:) ! sqrt g . (curl B)^s
   jireal = dBs(1,3,:) - dBs(3,1,:) ! sqrt g . (curl B)^t

   call tfft( Nt,Nz,ijreal,jireal,isr,trigm,trign,trigwk,mn,im,in,efmn(1:mn),ofmn(1:mn),cfmn(1:mn),sfmn(1:mn),ifail )
   
  !error(1) = error(1) + sum( ( ofmn(1:mn) - mu(ll)*( - im(1:mn)*Azmn(0,1:mn) - in(1:mn)*Atmn(0,1:mn) ) )**2 ) ! err in Fourier; (sqrt g . j^s)_mn - (sqrt g . B^s)_mn
  !error(2) = error(2) + sum( ( cfmn(1:mn) - mu(ll)*(                         -          Azmn(1,1:mn) ) )**2 ) ! err in Fourier;
   
   jerror(1) = jerror(1) + sum( abs( ofmn(1:mn) - mu(ll)*( - im(1:mn)*Azmn(0,1:mn) - in(1:mn)*Atmn(0,1:mn) ) ) ) ! err in Fourier; (sqrt g . j^s)_mn - (sqrt g . B^s)_mn
   jdenom(1) = jdenom(1) + sum( abs(            - mu(ll)*( - im(1:mn)*Azmn(0,1:mn) - in(1:mn)*Atmn(0,1:mn) ) ) ) ! err in Fourier; (sqrt g . j^s)_mn - (sqrt g . B^s)_mn

   jerror(2) = jerror(2) + sum( abs( cfmn(1:mn) - mu(ll)*(                         -          Azmn(1,1:mn) ) ) ) ! err in Fourier;
   jdenom(2) = jdenom(2) + sum( abs(            - mu(ll)*(                         -          Azmn(1,1:mn) ) ) ) ! err in Fourier;
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
   ijreal = dBs(2,1,:)-dBs(1,2,:) ! sqrt g . (curl B)^zz
   jireal = zero

   call tfft( Nt,Nz,ijreal,jireal,isr,trigm,trign,trigwk,mn,im,in,efmn(1:mn),ofmn(1:mn),cfmn(1:mn),sfmn(1:mn),ifail )

  !error(3) = error(3) + sum( ( efmn(1:mn) - mu(ll)*( +          Atmn(1,1:mn)                         ) )**2 ) ! err in Fourier; B^zz

   jerror(3) = jerror(3) + sum( abs( efmn(1:mn) - mu(ll)*( +          Atmn(1,1:mn)                         ) ) ) ! err in Fourier; B^zz
   jdenom(3) = jdenom(3) + sum( abs(            - mu(ll)*( +          Atmn(1,1:mn)                         ) ) ) ! err in Fourier; B^zz
   
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
   
  enddo ! end of do ii = 1 , lNi-1
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
 !error(1:3) = sqrt( error(1:3) / ( (lNi-1) * mn  ) )
  jerror(1:3) = jerror(1:3) / ( (lNi-1) * mn )
  jdenom(1:3) = jdenom(1:3) / ( (lNi-1) * mn )
  
  cput = MPI_WTIME() ; write(ounit,1000)cput-cpus,myid,ll,Ni(ll),jerror(1:3),jerror(1:3)/jdenom(1:3),Nen,cput-cpui
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
1000 format("jo00aa : "f10.2" : myid="i3" ; ll="i3" ; Ni="i4" ; curl error "6es23.15 " ; Nen="i3" ; time="f8.2"s ;")

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!

  RETURN(jo00aa)
  
!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
  
end subroutine jo00aa

!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!-!
